﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VTFC
{
    // ----------------------------------------------------------------------------
    /// <summary>
    /// New list/folder form.
    /// </summary>
    // ----------------------------------------------------------------------------
    public partial class FrmList : Form
    {
        // ----------------------------------------------------------------------------
        /// <summary>
        /// Constructor.
        /// </summary>
        // ----------------------------------------------------------------------------
        public FrmList()
        {
            InitializeComponent();
            InitializeContent();
        }//ctor

        private void InitializeContent()
        {
            this.Text = Resources.FrmList_Text;
            lblName.Text = Resources.FrmList_lblName;
            lblDescription.Text = Resources.FrmList_lblDescription;
            lblEnableVersioning.Text = Resources.FrmList_lblEnableVersioning;
            lblQuickLaunch.Text = Resources.FrmList_lblQuickLaunch;

            btnOk.Text = Resources.ButtonOk;
            btnCancel.Text = Resources.ButtonCancel;
        }//func InitializeContent

        // ----------------------------------------------------------------------------
        /// <summary>
        /// Name of the list.
        /// </summary>
        // ----------------------------------------------------------------------------
        public string ListName
        {
            get { return txtName.Text.Trim(); }
        }//prop ListName

        // ----------------------------------------------------------------------------
        /// <summary>
        /// Description of the list.
        /// </summary>
        // ----------------------------------------------------------------------------
        public string Description
        {
            get { return txtDescription.Text.Trim(); }
        }//prop Description

        // ----------------------------------------------------------------------------
        /// <summary>
        /// Enable versioning for the list.
        /// </summary>
        // ----------------------------------------------------------------------------
        public bool EnableVersioning
        {
            get { return ckbEnableVersioning.Checked; }
        }//prop EnableVersioning

        // ----------------------------------------------------------------------------
        /// <summary>
        /// Hide the list so that it does not appear on the Documents and Lists page, <br/> 
        /// Quick Launch bar, Modify Site Content page, or Add Column page as an option <br/> 
        /// for lookup fields.
        /// </summary>
        // ----------------------------------------------------------------------------
        public bool QuickLaunch
        {
            get { return ckbHidden.Checked; }
        }//prop QuickLaunch

        private bool ValidateData()
        {
            bool valid = true;
            if (ListName.Length == 0)
            {
                valid = false;
                BaseLogic.ShowError(Resources.Msg_Warrning_EmptyName);
            }
            return valid;
        }//func ValidateData

        private void Save()
        {
            if (ValidateData())
            {
                this.DialogResult = DialogResult.OK;
            }
        }//func Save

        private void btnOk_Click(object sender, EventArgs e)
        {
            Save();
        }//func btnOk_Click

        private void txtName_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyData)
            {
                case Keys.Enter:
                    Save();
                    break;
            }
        }//func txtName_KeyDown
    }//class FrmNewList
}//namespace VTFC
